"""This module contains all of the tools necessary to make Ren'Py
    visual novel distribution less of a hassle.

The overall premise of this module is to provide utilities that make
distribution of Ren'Py visual novels, particularly for macOS, easier.
"""
__version__ = '0.2.2'

from renpy_distribute_tools.fixed_zipfile import MyZipFile
from renpy_distribute_tools.plist import fix_plist
from renpy_distribute_tools.apple import package_app_zip, \
                                         upload_to_notary, build_pkg, staple, code_sign
from renpy_distribute_tools.util import file_exists, find_mac_build, verify_built_files



"""This module contains important utilities to make signing, notarizing, and
    building packages for macOS easier.

    Note: These functions require macOS and Xcode 10 or higher.
"""
import os
import subprocess
from sys import platform
from functools import wraps

def darwin_only(call):
    """A decorator for macOS-specific commands.

    This should be used to denote that a function only works on
    macOS due to reliance on built-in tools from macOS or Xcode.
    """
    @wraps(call)
    def darwin_call(*args, **kwargs):
        if platform.lower() != "darwin":
            raise OSError("Function %s only works on macOS." % (call))
        call(*args, **kwargs)
    return darwin_call

@darwin_only
def package_app_zip(app: str):
    """Create a ZIP file of the app.

        Args:
            app: The path to the macOS to make an archive of
    """
    if os.path.isdir(app):
        zip_commands = ["ditto", "-c", "-k", "--rsrc", "--keepParent", app, app + ".zip"]
        subprocess.check_call(zip_commands)
    else:
        raise NotADirectoryError("The .app file is either missing or not present.")

@darwin_only
def build_pkg(app: str, identity: str, package_name: str):
    """Create an installable package from a macOS app.

    By default, it will create an app package that installs to `/Applications/`.
    This package installer can also be used to submit an app to the Mac App Store.

    If the package name isn't a file path, `.pkg` will automatically be appended
    at the end of the name.

    Args:
        app: The path to the app to create a package of.
        identity: The identity to sign the package with
        package_name: The name or path of the resulting package.
    """
    package_file = package_name
    if ".pkg" not in package_name:
        package_file = package_name + ".pkg"
    commands = ["productbuild", "--component", app,
                "/Applications", "--sign", identity, package_file]
    return subprocess.check_call(commands)

@darwin_only
def code_sign(identity: str,
              app_directory: str,
              entitlements: str = None,
              enable_hardened_runtime: bool = False):
    """Digitally sign a macOS application with a signing identity and any entitlements.

    Args:
        identity: The identity to use during signing, usually a Developer ID
        app_directory: The path to the macOS application for signing
        entitlements: (Optional) The path to the entitlements the app should be signed with
    """
    commands = ["codesign",
                "--timestamp",
                "--deep",
                "--force",
                "--no-strict",
                "--sign",
                identity,
                app_directory]

    if entitlements is not None:
        commands.append("--entitlements")
        commands.append(entitlements)

    if enable_hardened_runtime:
        commands.append("--options=runtime")

    return subprocess.check_call(commands)

@darwin_only
def upload_to_notary(app: str, identifier: str, username: str, password: str, provider: str = None):
    """Upload a macOS application archive to Apple's notary service for notarization.

    Args:
        app: The path to the macOS application to send to Apple.
        identifier: The bundle identifier of the application.
        username: The username (email address) of the Apple ID to notarize under.
        password: The password of the Apple ID to notarize under.
        provider: (Optional) The App Store Connect or iTunes Connect provider
    associated with the Apple ID.
    """
    package_app_zip(app)
    commands = ["xcrun", "altool", "-t", "osx", "-f", app + ".zip",
                "--notarize-app", "--primary-bundle-id", identifier,
                "-u", username, "-p", password]

    if provider is not None:
        commands.append("-itc_provider")
        commands.append(provider)

    subprocess.check_call(commands)
    os.remove(app + ".zip")

@darwin_only
def staple(app: str):
    """Staple a notarization ticket to a notarized app.

    Args:
        app: The path of the macOS app to staple the ticket to.
    """
    commands = ["xcrun", "stapler", "staple", app]
    return subprocess.check_call(commands)



"""
    This module contains some utilities that help fix the property lists
    in the macOS versions of a Ren'Py project.
"""
import plistlib

# noinspection PyDeprecation
def fix_plist(plist_file: str, identifier: str, p_copyright: str):
    """Add the bundle identifier and copyright text to a Ren'Py-built macOS app's Info.plist.

    Args:
        plist_file: The path to the plist file to modify
        identifier: The bundle identifier to set the app to
        p_copyright: The human-readable copyright text to attach
    """
    plist = plistlib.readPlist(plist_file)
    plist['CFBundleIdentifier'] = identifier
    plist['NSHumanReadableCopyright'] = p_copyright
    plistlib.writePlist(plist, plist_file)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['renpy_distribute_tools']

package_data = \
{'': ['*']}

install_requires = \
['pylint>=2.4.4,<3.0.0']

setup_kwargs = {
    'name': 'renpy-distribute-tools',
    'version': '0.3.1',
    'description': "Utilities that make Ren'Py distribution less of a pain in the arse",
    'long_description': "# Ren'Py Distribution Tools\n\nA set of tools to make Ren'Py distribution less of a pain in the arse.\n\n[![MPL](https://img.shields.io/github/license/alicerunsonfedora/renpy-distribute-tools)](LICENSE.txt) \n![Python](https://img.shields.io/badge/python-3.7+-blue.svg) \n\n## Installing `renpy-distribute-tools`\n\nTo install via PyPI:\n\n```\npip install renpy-distribute-tools\n```\n\nOr, if you're using a Poetry project, just add the dependency:\n\n```\npoetry add renpy-distribute-tools\n```\n\n## Building from source\n\nRen'Py Distribute Tools is a Poetry project and can be built using Poetry's `build` command.\n\n1. Clone the repository.\n2. In the root of the project, run `poetry install`.\n3. Finally, run `poetry build`.\n\n## What's included\n\nThe Ren'Py Distribution Tools set comes with utilities that make it easy to do the following:\n\n- Modifying a visual novel's `Info.plist`.\n- Code-signing the visual novel binaries in the Mac app with entitlements.\n- Creating a ZIP copy of the Mac app and sending it to Apple's notarization servers.\n- Stapling the notarization ticket to a macOS app.\n\n## Usage\n\nSee the [documentation](https://alicerunsonfedora.github.io/renpy-distribute-tools) for more info.",
    'author': 'Marquis Kurt',
    'author_email': 'software@marquiskurt.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/alicerunsonfeodra/renpy-distribute-tools',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)



"""
    This module contains some basic utilities to locate Ren'Py
    distributables and to find the macOS version of a VN.
"""
import os

def file_exists(directory: str = os.getcwd(), item: str = ''):
    """Determine whether a file exists in a certain directory.

    Args:
        dir: The directory to search in.
        item: The item to search for in the directory.
    Returns:
        File name if it's found or `None` if it doesn't find anything
    """
    cwd = os.listdir(directory)
    for listed_item in cwd:
        if item in listed_item:
            return listed_item
    return None


def verify_built_files(directory: str = os.getcwd()):
    """Determine if the Ren'Py distributions have been built already by looking
    for the `-dists` directory.

    This function will check if the directory exists in itself.

    Args:
        dir: The directory to search.
    """
    return file_exists(directory=directory, item="-dists")


def find_mac_build(directory: str):
    """Determine whether the macOS builds have been created.

    Args:
        dir: The directory to search in
    """
    return file_exists(directory=directory, item="-mac")
